'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _camelCase = require('lodash/camelCase');

var _camelCase2 = _interopRequireDefault(_camelCase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Recursively initializes a components hierarchy
 *
 * @param  {Object|Function} component        [Wrapper component|Leaf component]
 * @param  {Object}          styles           [Styles for every component]
 * @param  {Object}          parentComponents [Already initialized components from parent layers]
 * @param  {Array}           keyPath          [Path to current component]
 * @return {Object}                           [Initialized component or components hierarchy]
 */
var initComponent = function initComponent(component, styles, parentComponents) {
  var keyPath = arguments.length <= 3 || arguments[3] === undefined ? [] : arguments[3];

  if (typeof component !== 'function') {
    return Object.keys(component).reduce(function (result, componentKey) {
      return _extends({}, result, _defineProperty({}, componentKey, initComponent(component[componentKey], styles, parentComponents, [].concat(_toConsumableArray(keyPath), [componentKey]))));
    }, {});
  } else {
    return component(styles[(0, _camelCase2.default)(keyPath[0])] || {}, parentComponents);
  }
};

/**
 * Initializes layer
 *
 * @param  {Object} layer            [Components object related to current layer]
 * @param  {Object} styles           [Styles for every component]
 * @param  {Object} parentComponents [Already initialized components from parent layers]
 * @return {Object}                  [Initialized components for current layer]
 */
var initLayer = function initLayer(layer, styles, parentComponents) {
  return Object.keys(layer).reduce(function (layerComponents, componentKey) {
    return _extends({}, layerComponents, _defineProperty({}, componentKey, initComponent(layer[componentKey], styles, parentComponents, [componentKey])));
  }, {});
};

/**
 * Initializes React UI
 *
 * @param  {Object} styles   [Styles for every component]
 * @param  {Array}  uiLayers [Components grouped into layer objects]
 * @return {Object}          [Initialized components object that is ultimately used by clients]
 */
var init = function init() {
  for (var _len = arguments.length, uiLayers = Array(_len), _key = 0; _key < _len; _key++) {
    uiLayers[_key] = arguments[_key];
  }

  return function () {
    var styles = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    return uiLayers.reduce(function (initializedComponents, layer) {
      return _extends({}, initializedComponents, initLayer(layer, styles, initializedComponents));
    }, {});
  };
};

exports.default = init;